/*
 MasksToLayers.jsx
 version: 3.0
 author: Charles Bordenave (www.nabscripts.com)
 date: 31 Dec 2012
*/


function MasksToLayers()
{
	var masksToLayers = this;
	
	var utils = new MasksToLayersUtils();

	// infos
	this.scriptName = "MasksToLayers.jsx";	
	this.scriptVersion = "3.0";
	this.scriptTitle = "Masks To Layers";
	this.scriptCopyright = "Copyright (c) 2012 Charles Bordenave";
	this.scriptHomepage = "http://www.nabscripts.com";
	this.scriptDescription = {en: "This script places the masks of the selected layer on individual layers.\\r\\rFor each new layer, you can specify the blending mode, and you can choose to move the anchor point to the center of the mask.", fr:"Ce script place chaque masque du calque sélectionné sur un calque séparé.\\r\\rPour chaque nouveau calque, il est possible de préciser un mode de fusion, et de choisir de déplacer le point d\\'ancrage au centre du masque."};
	this.scriptAbout = {en:this.scriptName + ", v" + this.scriptVersion + "\\r" + this.scriptCopyright + "\\r" + this.scriptHomepage + "\\r\\r" + utils.loc(this.scriptDescription), 
						fr:this.scriptName + ", v" + this.scriptVersion + "\\r" + this.scriptCopyright + "\\r" + this.scriptHomepage + "\\r\\r" + utils.loc(this.scriptDescription)};		
	this.scriptUsage = {en:	"\u25BA Select a layer having at least two masks \\r" +
							"\u25BA Specify the blending mode of the new layers and whether you want to move the layer\\\'s anchor point to the center of the mask \\r" +
							"\u25BA Click on Proceed",
						fr:	"\u25BA Sélectionnez un calque contenant au moins deux masques \\r" +
							"\u25BA Spécifiez le mode de fusion des nouveaux calques et si le point d\\\'ancrage doit être déplacé au centre du masque \\r" +
							"\u25BA Cliquez sur Exécuter"};
							
	// errors
	this.noCompErr = {en:"A comp must be active.", fr:"Une composition doit être active."};
	this.noLayersErr = {en:"Select a layer first.", fr:"Sélectionnez d'abord un calque."};

	// UI strings 
	this.aboutBtnName = "?";
	this.optionsPnlName = {en:"Options", fr:"Options"};
	this.blendingModeStName = {en:"Blending Mode:", fr:"Mode de fusion:"};
	this.blendingModeLstChoices = {en:"['Normal','Add','Alpha Add']", fr:"['Normal','Ajout','Ajout Alpha']"};
	this.anchorPointStName = {en:"Anchor Point:", fr:"Point d\\'ancrage:"};
	this.anchorPointLstChoices = {en:"['Do not change','Move To Mask Center']", fr:"['Ne pas changer','Déplacer au centre du masque']"};
	this.runBtnName = {en:"Proceed", fr:"Exécuter"};
	
	
	this.buildUI = function (thisObj)
	{
		// dockable panel or palette
		var pal = (thisObj instanceof Panel) ? thisObj : new Window("palette", this.scriptTitle, undefined, {resizeable:false});

		// resource specifications
		var res =
		"group { orientation:'column', alignment:['left','top'], alignChildren:['right','top'], \
			gr1: Group { \
				aboutBtn: Button { text:'" + this.aboutBtnName + "', preferredSize:[25,20] } \
			}, \
			gr2: Panel { orientation:'column', alignment:['fill','fill'], alignChildren:['right','top'], text:'" + utils.loc(this.optionsPnlName) + "', \
				gr21: Group { orientation:'row', \
					blendingModeSt: StaticText { text:'" + utils.loc(this.blendingModeStName) + "', value:true }, \
					blendingModeLst: DropDownList { properties:{items:" + utils.loc(this.blendingModeLstChoices) + "}, preferredSize:[150,20] } \
				}, \
				gr22: Group { orientation:'row', \
					anchorPointSt: StaticText { text:'" + utils.loc(this.anchorPointStName) + "', value:true }, \
					anchorPointLst: DropDownList { properties:{items:" + utils.loc(this.anchorPointLstChoices) + "}, preferredSize:[150,20] } \
				} \
			}, \
			gr3: Group { orientation:'row', alignment:['fill','top'], \
				runBtn: Button { text:'" + utils.loc(this.runBtnName) + "', alignment:['right','center'] } \
			} \
		}"; 
		pal.gr = pal.add(res);
		
		pal.gr.gr2.gr21.blendingModeLst.selection = 0;
		pal.gr.gr2.gr22.anchorPointLst.selection = 0;
		
		// event callbacks
		pal.gr.gr1.aboutBtn.onClick = function () 
		{ 
			utils.createAboutDlg(masksToLayers.scriptAbout, masksToLayers.scriptUsage); 
		};
		
		pal.gr.gr3.runBtn.onClick = function () 
		{ 
			masksToLayers.distributeMasks(pal); 
		};		
				
		// show user interface
		if (pal instanceof Window)
		{
			pal.center();
			pal.show();
		}
		else
		{
			pal.layout.layout(true);
		}	   
	};

	// Determines whether the active item is a composition  
	this.checkActiveItem = function () 
	{
		return !(app.project.activeItem instanceof CompItem);
	};   
	
	// Removes all masks except one
	this.removeLayerMasksExceptOne = function (layer, idToKeep)
	{
		for (var id = layer.Masks.numProperties; id > 0; id--)
		{
			if (id != idToKeep)
			{
				layer.Masks.property(id).remove();
			}
		}
	};

	// Repositions the anchor point around the center of the first mask
	this.moveAnchorPointToMaskCenter = function (layer)
	{
		var curTime = layer.containingComp.time;
		var shape = layer.Masks.property(1).maskShape.valueAtTime(curTime, false);
		var verts = shape.vertices;
		
		// Compute centroid
		var centroid = utils.getCenterOfMass(verts);
		var x = centroid[0];
		var y = centroid[1];
		var z = 0;
		
		// Reposition anchor point and adjust position
		var anchPt = layer.anchorPoint;
		var pos = layer.position;
		var offset = [x,y,z] - anchPt.valueAtTime(curTime, false);
		var newAnchPt = [x,y,z];
		var newPos = pos.valueAtTime(curTime, false) + offset;				
		
		anchPt.numKeys ? anchPt.setValueAtTime(curTime, newAnchPt) : anchPt.setValue(newAnchPt);
		pos.numKeys ? pos.setValueAtTime(curTime, newPos) : pos.setValue(newPos);
	};
		
	// Functional part of the script: places each mask of the selected layer on its own layer
	this.distributeMasks = function (pal)
	{
		try
		{
			var comp = app.project.activeItem;
			var err = this.noCompErr;
			if (this.checkActiveItem(comp)) throw(err);
					
			var err = this.noLayersErr;
			if (comp.selectedLayers.length < 1) throw(err);
			
			var layer = comp.selectedLayers[0];
			
			var blendingModeOption = pal.gr.gr2.gr21.blendingModeLst.selection.index;
			var anchorPointOption = pal.gr.gr2.gr22.anchorPointLst.selection.index;  
						
			app.beginUndoGroup(this.scriptTitle);
				  
			for (var maskId = 1; maskId <= layer.Masks.numProperties; maskId++) 
			{
				var dupLayer = layer.duplicate();
				
				this.removeLayerMasksExceptOne(dupLayer, maskId);
				
				dupLayer.name = (layer.name + " - " + layer.Masks.property(maskId).name).substring(0,31);			
				
				dupLayer.audioEnabled = false;
				
				switch (blendingModeOption)
				{
					case 0: break;
					case 1: dupLayer.blendingMode = BlendingMode.ADD; break;
					case 2: dupLayer.blendingMode = BlendingMode.ALPHA_ADD; break;
					default: break;
				}
				
				if (anchorPointOption == 1)
				{
					this.moveAnchorPointToMaskCenter(dupLayer);
				}				
			}
			
			layer.enabled = false;
				  
			app.endUndoGroup();
		}
		catch(err)
		{
			utils.throwErr(err);
		}				
	};
	
	this.run = function (thisObj) 
	{
		this.buildUI(thisObj);
	};
}


// This class provides some utility functions
function MasksToLayersUtils()
{
	var utils = this;
	
	this.loc = function (str)
	{
		var lang = parseFloat(app.version) < 9 ? $.locale : app.isoLanguage;
		return lang.toLowerCase().match("fr") ? str.fr : str.en;
	};

	this.throwErr = function (err)
	{
		var title = $.fileName.substring($.fileName.lastIndexOf("/")+1, $.fileName.lastIndexOf("."));
		alert(this.loc(err), title, true);
	};
	
	this.createAboutDlg = function (aboutStr, usageStr)
	{	
		eval(unescape('%09%09%76%61%72%20%64%6C%67%20%3D%20%6E%65%77%20%57%69%6E%64%6F%77%28%22%64%69%61%6C%6F%67%22%2C%20%22%41%62%6F%75%74%22%29%3B%0A%09%20%20%20%20%20%20%09%20%20%20%20%20%20%20%09%0A%09%20%20%20%20%76%61%72%20%72%65%73%20%3D%0A%09%09%22%67%72%6F%75%70%20%7B%20%6F%72%69%65%6E%74%61%74%69%6F%6E%3A%27%63%6F%6C%75%6D%6E%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%66%69%6C%6C%27%2C%27%66%69%6C%6C%27%5D%2C%20%61%6C%69%67%6E%43%68%69%6C%64%72%65%6E%3A%5B%27%66%69%6C%6C%27%2C%27%66%69%6C%6C%27%5D%2C%20%5C%0A%09%09%09%70%6E%6C%3A%20%50%61%6E%65%6C%20%7B%20%74%79%70%65%3A%27%74%61%62%62%65%64%70%61%6E%65%6C%27%2C%20%5C%0A%09%09%09%09%61%62%6F%75%74%54%61%62%3A%20%50%61%6E%65%6C%20%7B%20%74%79%70%65%3A%27%74%61%62%27%2C%20%74%65%78%74%3A%27%44%65%73%63%72%69%70%74%69%6F%6E%27%2C%20%5C%0A%09%09%09%09%09%61%62%6F%75%74%45%74%3A%20%45%64%69%74%54%65%78%74%20%7B%20%74%65%78%74%3A%27%22%20%2B%20%74%68%69%73%2E%6C%6F%63%28%61%62%6F%75%74%53%74%72%29%20%2B%20%22%27%2C%20%70%72%65%66%65%72%72%65%64%53%69%7A%65%3A%5B%33%36%30%2C%32%30%30%5D%2C%20%70%72%6F%70%65%72%74%69%65%73%3A%7B%6D%75%6C%74%69%6C%69%6E%65%3A%74%72%75%65%7D%20%7D%20%5C%0A%09%09%09%09%7D%2C%20%5C%0A%09%09%09%09%75%73%61%67%65%54%61%62%3A%20%50%61%6E%65%6C%20%7B%20%74%79%70%65%3A%27%74%61%62%27%2C%20%74%65%78%74%3A%27%55%73%61%67%65%27%2C%20%5C%0A%09%09%09%09%09%75%73%61%67%65%45%74%3A%20%45%64%69%74%54%65%78%74%20%7B%20%74%65%78%74%3A%27%22%20%2B%20%74%68%69%73%2E%6C%6F%63%28%75%73%61%67%65%53%74%72%29%20%2B%20%22%27%2C%20%70%72%65%66%65%72%72%65%64%53%69%7A%65%3A%5B%33%36%30%2C%32%30%30%5D%2C%20%70%72%6F%70%65%72%74%69%65%73%3A%7B%6D%75%6C%74%69%6C%69%6E%65%3A%74%72%75%65%7D%20%7D%20%5C%0A%09%09%09%09%7D%20%5C%0A%09%09%09%7D%2C%20%5C%0A%09%09%09%62%74%6E%73%3A%20%47%72%6F%75%70%20%7B%20%6F%72%69%65%6E%74%61%74%69%6F%6E%3A%27%72%6F%77%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%66%69%6C%6C%27%2C%27%62%6F%74%74%6F%6D%27%5D%2C%20%5C%0A%09%09%09%09%6F%74%68%65%72%53%63%72%69%70%74%73%42%74%6E%3A%20%42%75%74%74%6F%6E%20%7B%20%74%65%78%74%3A%27%4F%74%68%65%72%20%53%63%72%69%70%74%73%2E%2E%2E%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%6C%65%66%74%27%2C%27%63%65%6E%74%65%72%27%5D%20%7D%2C%20%5C%0A%09%09%09%09%6F%6B%42%74%6E%3A%20%42%75%74%74%6F%6E%20%7B%20%74%65%78%74%3A%27%4F%6B%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%72%69%67%68%74%27%2C%27%63%65%6E%74%65%72%27%5D%20%7D%20%5C%0A%09%09%09%7D%20%5C%0A%09%09%7D%22%3B%20%0A%09%09%64%6C%67%2E%67%72%20%3D%20%64%6C%67%2E%61%64%64%28%72%65%73%29%3B%0A%09%09%0A%09%09%64%6C%67%2E%67%72%2E%70%6E%6C%2E%61%62%6F%75%74%54%61%62%2E%61%62%6F%75%74%45%74%2E%6F%6E%43%68%61%6E%67%65%20%3D%20%64%6C%67%2E%67%72%2E%70%6E%6C%2E%61%62%6F%75%74%54%61%62%2E%61%62%6F%75%74%45%74%2E%6F%6E%43%68%61%6E%67%69%6E%67%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%0A%09%09%7B%0A%09%09%09%74%68%69%73%2E%74%65%78%74%20%3D%20%75%74%69%6C%73%2E%6C%6F%63%28%61%62%6F%75%74%53%74%72%29%2E%72%65%70%6C%61%63%65%28%2F%5C%5C%72%2F%67%2C%20%27%5C%72%27%29%3B%0A%09%09%7D%3B%0A%09%09%0A%09%09%64%6C%67%2E%67%72%2E%70%6E%6C%2E%75%73%61%67%65%54%61%62%2E%75%73%61%67%65%45%74%2E%6F%6E%43%68%61%6E%67%65%20%3D%20%64%6C%67%2E%67%72%2E%70%6E%6C%2E%75%73%61%67%65%54%61%62%2E%75%73%61%67%65%45%74%2E%6F%6E%43%68%61%6E%67%69%6E%67%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%0A%09%09%7B%0A%09%09%09%74%68%69%73%2E%74%65%78%74%20%3D%20%75%74%69%6C%73%2E%6C%6F%63%28%75%73%61%67%65%53%74%72%29%2E%72%65%70%6C%61%63%65%28%2F%5C%5C%72%2F%67%2C%20%27%5C%72%27%29%2E%72%65%70%6C%61%63%65%28%2F%5C%5C%27%2F%67%2C%20%22%27%22%29%3B%0A%09%09%7D%3B%0A%09%09%09%0A%09%09%64%6C%67%2E%67%72%2E%62%74%6E%73%2E%6F%74%68%65%72%53%63%72%69%70%74%73%42%74%6E%2E%6F%6E%43%6C%69%63%6B%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%0A%09%09%7B%0A%09%09%09%76%61%72%20%63%6D%64%20%3D%20%22%22%3B%0A%09%09%09%76%61%72%20%75%72%6C%20%3D%20%22%68%74%74%70%3A%2F%2F%61%65%73%63%72%69%70%74%73%2E%63%6F%6D%2F%61%75%74%68%6F%72%73%2F%6D%2D%70%2F%6E%61%62%2F%22%3B%0A%09%0A%09%09%09%69%66%20%28%24%2E%6F%73%2E%69%6E%64%65%78%4F%66%28%22%57%69%6E%22%29%20%21%3D%20%2D%31%29%0A%09%09%09%7B%0A%09%20%20%20%20%20%20%20%20%09%69%66%20%28%46%69%6C%65%28%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%22%29%2E%65%78%69%73%74%73%29%0A%09%09%09%09%09%63%6D%64%20%2B%3D%20%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%20%22%20%2B%20%75%72%6C%3B%0A%09%09%09%09%65%6C%73%65%20%69%66%20%28%46%69%6C%65%28%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%20%28%78%38%36%29%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%22%29%2E%65%78%69%73%74%73%29%0A%09%09%09%09%09%63%6D%64%20%2B%3D%20%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%20%28%78%38%36%29%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%20%22%20%2B%20%75%72%6C%3B%0A%09%09%09%09%65%6C%73%65%0A%09%09%09%09%09%63%6D%64%20%2B%3D%20%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%49%6E%74%65%72%6E%65%74%20%45%78%70%6C%6F%72%65%72%2F%69%65%78%70%6C%6F%72%65%2E%65%78%65%20%22%20%2B%20%75%72%6C%3B%0A%09%09%09%7D%0A%09%09%09%65%6C%73%65%0A%09%09%09%09%63%6D%64%20%2B%3D%20%22%6F%70%65%6E%20%5C%22%22%20%2B%20%75%72%6C%20%2B%20%22%5C%22%22%3B%20%20%20%20%20%20%20%20%20%0A%09%09%09%74%72%79%0A%09%09%09%7B%0A%09%09%09%09%73%79%73%74%65%6D%2E%63%61%6C%6C%53%79%73%74%65%6D%28%63%6D%64%29%3B%0A%09%09%09%7D%0A%09%09%09%63%61%74%63%68%28%65%29%0A%09%09%09%7B%0A%09%09%09%09%61%6C%65%72%74%28%65%29%3B%0A%09%09%09%7D%0A%09%09%7D%3B%0A%09%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%0A%09%09%64%6C%67%2E%67%72%2E%62%74%6E%73%2E%6F%6B%42%74%6E%2E%6F%6E%43%6C%69%63%6B%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%20%0A%09%09%7B%0A%09%09%09%64%6C%67%2E%63%6C%6F%73%65%28%29%3B%20%0A%09%09%7D%3B%0A%09%20%20%20%20%20%20%20%0A%09%09%64%6C%67%2E%63%65%6E%74%65%72%28%29%3B%0A%09%09%64%6C%67%2E%73%68%6F%77%28%29%3B'));
	};		

	// Computes area of a polygon defined by a set of points 
	this.getArea = function (points)
	{
		var area = 0;	
		for (var i = 0; i < points.length; i++) 
		{
			var j = (i + 1) % points.length;
			area += points[i][0] * points[j][1];
			area -= points[i][1] * points[j][0];
		}
		area /= 2.0;	
		return area; 
	};
	
	this.getCenterOfMass = function (points)
	{
		var cx = 0, cy = 0;
		var factor = 0;
		var A = this.getArea(points);
		for (var i = 0; i < points.length; i++) 
		{
			var j = (i + 1) % points.length;
			factor = points[i][0] * points[j][1] - points[j][0] * points[i][1];
			cx+= (points[i][0] + points[j][0]) * factor;
			cy+= (points[i][1] + points[j][1]) * factor;
		}
		A *= 6.0;
		factor = 1 / A;
		cx *= factor;
		cy *= factor;
		return [cx,cy];
	};
}


// run script
new MasksToLayers().run(this);
